package biggests.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import biggests.tree.*;
import biggests.utils.*;
import smadeira.biclustering.*;

/**
 * <p>Title: Expression Panel</p>
 *
 * <p>Description: Displays expression charts. May contain bicluster
 *                 time-points, all time-points or bicluster pattern
 *                 charts. The charts are constructed from either single
 *                 biclusters or groups of biclusters.</p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana Gonalves
 */
public class PanelExpression extends JPanel {
    /**
     * Constant which defines bicluster time-points expression chart type.
     */
    public static final int BICLUSTER_TIME_POINTS = 0;

    /**
     * Constant which defines all time-points expression chart type.
     */
    public static final int ALL_TIME_POINTS = 1;

    /**
     * Constant which defines pattern expression chart type.
     */
    public static final int PATTERN = 2;

    /**
     * Visualization options (number of biclusters to be displayed) to
     * include in a combo box when the expression type is either
     * BICLUSTER_TIME_POINTS or PATTERN and the corresponding object is a
     * group of biclusters.
     */
    private static final String[] visualizationOption = {
            "first 12 biclusters",
            "first 24 biclusters",
            "first 48 biclusters",
            "first 96 biclusters",
            "all biclusters"};

    /**
     * The type of expression for this panel's chart. It may be:<br/>
     * PanelExpression.BICLUSTER_TIME_POINTS
     * PanelExpression.ALL_TIME_POINTS
     * PanelExpression.PATTERN
     */
    private int expressionType;

    /**
     * The application's main frame, where this panel is being displayed.
     */
    private BiggestsFrame mainFrame;

    /**
     * Check box which allows the user to choose between an expression
     * chart with original or normalized data. To be used on
     * BICLUSTER_TIME_POINTS and ALL_TIME_POINTS expression charts panel.
     */
    private JCheckBox normalize;

    /**
     * Combo box for the user to choose the number of biclusters to display.
     * To be used when the expression type is either BICLUSTER_TIME_POINTS
     * or PATTERN and the corresponding object is a group of biclusters.
     */
    private JComboBox option;

    private JPanel optionPanel;

    /**
     * The expression chart of a single bicluster. It can be a bicluster
     * time-points, an all time-points or a pattern expression chart.
     */
    private PanelExpressionChart expressionChart;

    /**
     * The expression charts of a set of biclusters (which belong to a
     * specific group of biclusters).
     */
    private PanelExpressionChart[] expressionCharts;

    /**
     * The set of biclusters retrieved from the group of biclusters to display.
     */
    private Bicluster[] setOfBiclusters;

    /**
     * The width of the expression chart.
     */
    private int chartWidth;

    /**
     * The height of the expression chart.
     */
    private int chartHeight;

    /**
     * Creates a new panel to display expression charts.
     * This panel may hold three different types of expression charts:<br/>
     * - <code>BICLUSTER_TIME_POINTS</code><br/>
     * - <code>ALL_TIME_POINTS</code><br/>
     * - <code>PATTERN</code><br/>
     *
     * @param frame the <code>BiggestsFrame</code>, where this panel is
     *              being displayed.
     * @param expressionType the type of expression chart(s) which is/are
     *                       being displayed in this panel
     */
    public PanelExpression(BiggestsFrame frame, int expressionType) {
        super();
        this.mainFrame = frame;
        this.expressionType = expressionType;
        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Initializes graphic components.
     *
     * @throws Exception when some component could not be initialized
     */
    private void jbInit() throws Exception {
        this.setLayout(new SpringLayout());
        this.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
        this.addHierarchyBoundsListener(new
                PanelExpression_this_hierarchyBoundsAdapter(this));
        this.normalize = new JCheckBox("Normalize to mean 0 and std 1");
        normalize.addItemListener(new PanelExpression_normalize_itemAdapter(this));
        this.optionPanel = new JPanel();
        optionPanel.setLayout(new SpringLayout());
        this.option = new JComboBox(visualizationOption);
        optionPanel.add(option);
        option.addItemListener(new PanelExpression_option_itemAdapter(this));
    }

    /**
     * Clears this panel. Removes the graphic components.
     */
    public void clear() {
        this.removeAll();
        this.setLayout(new SpringLayout());
        expressionChart = null;
        expressionCharts = null;
        setOfBiclusters = null;
    }

    /**
     * Sets this panels' expression chart width and height depending on the
     * specified number of genes and number of conditions of the bicluster
     * to analyse.
     *
     * @param numberOfGenes the <code>int</code> number of genes
     * @param numberOfConditions the <code>int</code> number of conditions
     */
    private void setChartWidthAndHeight(int numberOfGenes,
                                        int numberOfConditions) {
        if (numberOfConditions < 3) {
            chartWidth = 700;
        } else {
            chartWidth = 800;
        }

        if (numberOfGenes < 20) {
            chartHeight = 400;
        } else
        if (numberOfGenes < 50) {
            chartHeight = 500;
        } else
        if (numberOfGenes < 100) {
            chartHeight = 600;
        } else {
            int mult = numberOfGenes / 100;
            chartHeight = (mult - 1) * 150 + 700;
        }

        this.setPreferredSize(new Dimension(chartWidth + 40, chartHeight + 80));
    }

    /**
     * Paints an expression chart which data consists of the gene expression
     * values for the genes in a specific <code>nodeInfo</code> matrix object
     * under all conditions of a specific <code>parentNodeInfo</code> matrix
     * object.
     *
     * @param nodeInfo the <code>NodeInfo</code> with the matrix object to
     *                 retrieve the genes
     * @param parentNodeInfo the <code>NodeInfo</code> with the matrix object
     *                       to retrieve all the conditions
     */
    public void paintAllTimePointsChart(NodeInfo nodeInfo,
                                        NodeInfo parentNodeInfo) {
        this.setLayout(new SpringLayout());
        if ((expressionType != ALL_TIME_POINTS)) {
            try {
                throw new InvalidNodeObjectTypeException(
                        "This method is to use " +
                        "exclusively to draw 'all time points' charts.");
            } catch (InvalidNodeObjectTypeException ex) {
                ex.printStackTrace();
                return;
            }
        }

        NodeObjectInterface nodeObject = nodeInfo.getNodeObject();
        NodeObjectInterface parentNodeObject = parentNodeInfo.getNodeObject();

        if ((parentNodeObject instanceof CCC_Biclustering) &&
            (nodeObject instanceof CCC_Bicluster)) {
            CCC_Biclustering biclustering = (CCC_Biclustering) parentNodeObject;
            CCC_Bicluster bicluster = (CCC_Bicluster) nodeObject;
            int numberOfGenes = bicluster.getNumberOfGenes();
            int numberOfConditions = 0;
            numberOfConditions = ((DiscretizedExpressionMatrix) biclustering.
                                  getMatrix()).
                                 getOriginalExpressionMatrix().
                                 getNumberOfConditions();

            this.setChartWidthAndHeight(numberOfGenes, numberOfConditions);

            try {
                expressionChart = PanelExpressionChart.
                                  createExpressionAllTimePointsChart(
                                          bicluster,
                                          chartWidth,
                                          chartHeight,
                                          false);

                normalize.setSelected(false);
                this.add(normalize, 0);
                this.add(expressionChart, 1);
                ((SpringLayout)this.getLayout()).
                        putConstraint(SpringLayout.NORTH, expressionChart,
                                      20,
                                      SpringLayout.SOUTH, normalize);
            } catch (InvalidNodeObjectTypeException ex) {
                /**
                 * this exception is an internal error, it shouldn't be
                 * displayed to the user; besides, it does not happen, since
                 * application frame only sends valid node objects
                 */
                ex.printStackTrace();
                return;
            }
        } else
        if ((parentNodeObject instanceof E_CCC_Biclustering) &&
            (nodeObject instanceof E_CCC_Bicluster)) {
            E_CCC_Biclustering biclustering = (E_CCC_Biclustering)
                                              parentNodeObject;
            E_CCC_Bicluster bicluster = (E_CCC_Bicluster) nodeObject;
            int numberOfGenes = bicluster.getNumberOfGenes();
            int numberOfConditions = 0;
            numberOfConditions = ((DiscretizedExpressionMatrix)
                                  biclustering.getMatrix()).
                                 getOriginalExpressionMatrix().
                                 getNumberOfConditions();

            this.setChartWidthAndHeight(numberOfGenes, numberOfConditions);

            try {
                expressionChart = PanelExpressionChart.
                                  createExpressionAllTimePointsChart(
                                          bicluster,
                                          chartWidth,
                                          chartHeight,
                                          false);

                normalize.setSelected(false);
                this.add(normalize, 0);
                this.add(expressionChart, 1);
                ((SpringLayout)this.getLayout()).putConstraint(SpringLayout.NORTH, expressionChart,
                                           20,
                                           SpringLayout.SOUTH, normalize);
            } catch (InvalidNodeObjectTypeException ex) {
                /**
                 * this exception is an internal error, it shouldn't be
                 * displayed to the user; besides, it does not happen, since
                 * application frame only sends valid node objects
                 */
                ex.printStackTrace();
                return;
            }
        } else
        if ((parentNodeObject instanceof CC_TSB_Biclustering) &&
            (nodeObject instanceof Bicluster)) {
            CC_TSB_Biclustering biclustering = (CC_TSB_Biclustering)
                                               parentNodeObject;
            Bicluster bicluster = (Bicluster) nodeObject;
            int numberOfGenes = bicluster.getNumberOfGenes();
            int numberOfConditions = 0;
            numberOfConditions = biclustering.getMatrix().getNumberOfConditions();

            this.setChartWidthAndHeight(numberOfGenes, numberOfConditions);

            try {
                expressionChart = PanelExpressionChart.
                                  createExpressionAllTimePointsChart(
                                          bicluster,
                                          chartWidth,
                                          chartHeight,
                                          false);

                normalize.setSelected(false);
                this.add(normalize, 0);
                this.add(expressionChart, 1);
                ((SpringLayout)this.getLayout()).putConstraint(SpringLayout.NORTH, expressionChart,
                                           20,
                                           SpringLayout.SOUTH, normalize);
            } catch (InvalidNodeObjectTypeException ex) {
                /**
                 * this exception is an internal error, it shouldn't be
                 * displayed to the user; besides, it does not happen, since
                 * application frame only sends valid node objects
                 */
                ex.printStackTrace();
                return;
            }
        }

        else {
            /**
             * do the same for other types of biclusters/bicluster groups
             */
        }
    }

    /**
     * Paints one or a set of bicluster time-point charts. The expression
     * values to display on the graph(s) are retrieved from the
     * <code>NodeInfo</code> object.
     *
     * @param nodeInfo the <code>NodeInfo</code> which contains the matrix
     *                 object(s) with the expression values
     */
    public void paintBiclusterTimePointsCharts(NodeInfo nodeInfo) {
        this.paint(nodeInfo);
    }

    /**
     * Paints one bicluster / set of bicluster pattern charts. The
     * pattern values to display in the pattern chart are retrieved from
     * the <code>NodeInfo</code> object.
     *
     * @param nodeInfo the <code>NodeInfo</code> which contains the matrix
     *                 object(s) with the expression values
     */
    public void paintBiclusterPatternCharts(NodeInfo nodeInfo) {
        this.paint(nodeInfo);
    }

    /**
     * Paints biclusters time-point charts or biclusters pattern charts.
     * The number of charts displayed depends on the type of the
     * <code>NodeInfo</code> object. If it holds a group of biclusters,
     * the charts to display correspond to the biclusters in the group.
     * If it holds a single bicluster, only an expression / pattern
     * chart will be shown.
     *
     * @param nodeInfo the <code>NodeInfo</code> object with the data to
     *                 construct the expression/pattern charts
     */
    private void paint(NodeInfo nodeInfo) {
        int nodeObjectType = nodeInfo.getNodeObjectType();
        if ((nodeObjectType != NodeInfo.BICLUSTER_SINGLE) &&
            (nodeObjectType != NodeInfo.POSTPROCESSED_SINGLE) &&
            (nodeObjectType != NodeInfo.BICLUSTER_GROUP) &&
            (nodeObjectType != NodeInfo.POSTPROCESSED_GROUP)) {
            try {
                throw new InvalidNodeObjectTypeException("Only individual " +
                        "biclusters or biclusters' groups are allowed.");
            } catch (InvalidNodeObjectTypeException ex) {
                ex.printStackTrace();
                return;
            }
        }

        if (expressionType == ALL_TIME_POINTS) {
            try {
                throw new InvalidNodeObjectTypeException("This method is " +
                        "intended to draw bicluster time points or " +
                        "bicluster pattern charts only");
            } catch (InvalidNodeObjectTypeException ex) {
                ex.printStackTrace();
                return;
            }
        }

        if ((expressionType == PATTERN) &&
            (nodeInfo.getDataType() == NodeInfo.VALUES)) {
            try {
                throw new InvalidNodeObjectTypeException("Biclusters with " +
                        "float values do not have a discrete pattern.");
            } catch (InvalidNodeObjectTypeException ex) {
                ex.printStackTrace();
                return;
            }
        }

        if ((nodeObjectType == NodeInfo.BICLUSTER_SINGLE) ||
            (nodeObjectType == NodeInfo.POSTPROCESSED_SINGLE)) {
            /**
             * node information object contains an individual bicluster object
             */
            this.setLayout(new SpringLayout());
            Bicluster bicluster = (Bicluster) nodeInfo.getNodeObject();
            int numberOfGenes = bicluster.getNumberOfGenes();
            int numberOfConditions = bicluster.getNumberOfConditions();

            if (expressionType == PATTERN) {
                this.chartWidth = 700;
                this.chartHeight = 300;
                this.setSize(new Dimension(chartWidth + 40,
                                           chartHeight + 40));
                try {
                    expressionChart = PanelExpressionChart.
                                      createExpressionBiclusterPatternChart(
                                              (CCC_Bicluster) bicluster,
                                              chartWidth,
                                              chartHeight,
                                              false);
                    this.add(expressionChart);
                    this.resetSize();
                } catch (Exception ex) {
                    ex.printStackTrace();
                    return;
                }
                return ;
            }

            this.setChartWidthAndHeight(numberOfGenes, numberOfConditions);

            try {
                expressionChart = PanelExpressionChart.
                                  createExpressionBiclusterTimePointsChart(
                                          bicluster,
                                          chartWidth,
                                          chartHeight,
                                          false,
                                          false);
                normalize.setSelected(false);
                this.add(normalize, 0);
                this.add(expressionChart, 1);
                this.resetSize();
                ((SpringLayout)this.getLayout()).putConstraint(SpringLayout.NORTH,
                                           expressionChart, 20,
                                           SpringLayout.SOUTH, normalize);
            } catch (InvalidNodeObjectTypeException ex) {
                /**
                 * this exception is an internal error, it shouldn't be
                 * displayed to the user; besides, it does not happen, since
                 * we are checking and sending a valid object type
                 */
                ex.printStackTrace();
                return;
            }
        } else {
            /**
             * node information object has a bicluster group or post-processed
             * bicluster group
             */
            this.setLayout(new FlowLayout());
            chartWidth = 150;
            chartHeight = 150;

            Biclustering biclustering = (Biclustering) nodeInfo.getNodeObject();
            setOfBiclusters = biclustering.getSetOfBiclusters();
            int numberOfCharts = 12;
            if (setOfBiclusters.length < 12) {
                numberOfCharts = setOfBiclusters.length;
            }

            this.resetSize();

            expressionCharts = new PanelExpressionChart[numberOfCharts];

            if ((biclustering instanceof CCC_Biclustering) ||
                (biclustering instanceof E_CCC_Biclustering)) {
                /**
                 * biclustering object is either a CCC_Biclustering or a
                 * E_CCC_Biclustering
                 */

                if (expressionType == PATTERN) {
                    option.setSelectedIndex(0);
                    this.add(optionPanel, 0);
                    /**
                     * biclustering object is either a CCC_Biclustering or
                     * a E_CCC_Biclustering and the type of charts to draw
                     * is PATTERN
                     */
                    try {
                        if (nodeObjectType == NodeInfo.POSTPROCESSED_GROUP) {
                            for (int i = 0; i < numberOfCharts; i++) {
                                CCC_Bicluster bicluster = (CCC_Bicluster)
                                        setOfBiclusters[i];
                                expressionCharts[i] = PanelExpressionChart.
                                        createExpressionBiclusterPatternChart(
                                                bicluster,
                                                chartWidth,
                                                chartHeight,
                                                "Post-Processed " + (i + 1) +
                                                "     (" + bicluster.getID() +
                                                ")",
                                                true);
                                this.add(expressionCharts[i]);
                            }

                        } else {
                            for (int i = 0; i < numberOfCharts; i++) {
                                CCC_Bicluster bicluster = (CCC_Bicluster)
                                        setOfBiclusters[i];
                                expressionCharts[i] = PanelExpressionChart.
                                        createExpressionBiclusterPatternChart(
                                                bicluster,
                                                chartWidth,
                                                chartHeight,
                                                true);
                                this.add(expressionCharts[i]);
                            }
                        }
                    } catch (Exception ex) {
                        ex.printStackTrace();
                        return;
                    }
                } else {
                    /**
                     * biclustering object is either a CCC_Biclustering or
                     * a E_CCC_Biclustering and the type of charts to draw
                     * is BICLUSTER_TIME_POINTS
                     */
                    expressionCharts = new
                                       PanelExpressionChart[numberOfCharts];
                    option.setSelectedIndex(0);
                    this.add(optionPanel, 0);
                    try {
                        if (nodeObjectType == NodeInfo.POSTPROCESSED_GROUP) {
                            for (int i = 0; i < numberOfCharts; i++) {
                                CCC_Bicluster bicluster = (CCC_Bicluster)
                                        setOfBiclusters[i];
                                expressionCharts[i] = PanelExpressionChart.
                                        createExpressionBiclusterTimePointsChart(
                                                bicluster,
                                                chartWidth,
                                                chartHeight,
                                                "Post-Processed " + (i + 1) +
                                                "     (" + bicluster.getID() +
                                                ")",
                                                false,
                                                true);
                                this.add(expressionCharts[i]);
                            }
                        } else {
                            for (int i = 0; i < numberOfCharts; i++) {
                                CCC_Bicluster bicluster = (CCC_Bicluster)
                                        setOfBiclusters[i];
                                expressionCharts[i] = PanelExpressionChart.
                                        createExpressionBiclusterTimePointsChart(
                                                bicluster,
                                                chartWidth,
                                                chartHeight,
                                                false,
                                                true);
                                this.add(expressionCharts[i]);
                            }
                        }
                    } catch (InvalidNodeObjectTypeException ex) {
                        ex.printStackTrace();
                        return;
                    }
                }
                return;
            } else {
                /**
                 * biclustering object is a CC_TSB_Biclustering
                 */
                if (expressionType == BICLUSTER_TIME_POINTS) {
                    expressionCharts = new
                                       PanelExpressionChart[numberOfCharts];
                    option.setSelectedIndex(0);
                    this.add(optionPanel, 0);
                    try {
                        if (nodeObjectType == NodeInfo.POSTPROCESSED_GROUP) {
                            for (int i = 0; i < numberOfCharts; i++) {
                                Bicluster bicluster = (Bicluster)
                                        setOfBiclusters[i];
                                expressionCharts[i] = PanelExpressionChart.
                                        createExpressionBiclusterTimePointsChart(
                                                bicluster,
                                                chartWidth,
                                                chartHeight,
                                                "Post-Processed " + (i + 1) +
                                                "     (" + bicluster.getID() +
                                                ")",
                                                false,
                                                true);
                                this.add(expressionCharts[i]);
                            }
                        } else {
                            for (int i = 0; i < numberOfCharts; i++) {
                                Bicluster bicluster = (Bicluster)
                                        setOfBiclusters[i];
                                expressionCharts[i] = PanelExpressionChart.
                                        createExpressionBiclusterTimePointsChart(
                                                bicluster,
                                                chartWidth,
                                                chartHeight,
                                                false,
                                                true);
                                this.add(expressionCharts[i]);
                            }
                        }
                    } catch (InvalidNodeObjectTypeException ex) {
                        ex.printStackTrace();
                        return;
                    }
                }
            }
        }
    }

    /**
     * Displays a chart, within a set of charts from a biclustering object.
     * The charts of a set of charts are displayed in lines of four. This
     * method automatically calculates the position of the chart in the
     * panel based on its given index.
     *
     * @param charts the complete <code>PanelExpressionChart[]</code> list of charts
     * @param chart the <code>PanelExpressionChart</code> to display
     * @param index the <code>int</code> index of the chart to display
     */
    private void displayBiclusterChart(PanelExpressionChart[] charts,
                                       PanelExpressionChart chart,
                                       int index) {
        this.add(chart, index + 1);
        if (index == 0) {
            /**
             * this is the first chart to display, i.e, will appear on
             * the first line, first column
             */
            ((SpringLayout)this.getLayout()).putConstraint(SpringLayout.WEST, chart, 0,
                                       SpringLayout.WEST, option);
            ((SpringLayout)this.getLayout()).putConstraint(SpringLayout.NORTH, chart, 20,
                                       SpringLayout.SOUTH, option);
            return;
        }

        if (index % 4 == 0) {
            /**
             * this is not the first chart to display, but it is
             * on the first column; this means that this chart is the
             * first chart of one specific line
             */
            ((SpringLayout)this.getLayout()).putConstraint(SpringLayout.WEST, chart, 0,
                                       SpringLayout.WEST, charts[0]);
            ((SpringLayout)this.getLayout()).putConstraint(SpringLayout.NORTH, chart, 10,
                                       SpringLayout.SOUTH, charts[index - 4]);
            return;
        }

        if (index > 4) {
            /**
             * this chart is to be displayed, not on the first line
             * nor on the first column, but in any of other position
             */
            ((SpringLayout)this.getLayout()).putConstraint(SpringLayout.WEST, chart, 10,
                                       SpringLayout.EAST, charts[index - 1]);
            ((SpringLayout)this.getLayout()).putConstraint(SpringLayout.NORTH, chart, 10,
                                       SpringLayout.SOUTH, charts[index - 4]);
            return;
        }

        /**
         * this chart is not the first chart, but it is on the first line
         */
        ((SpringLayout)this.getLayout()).putConstraint(SpringLayout.WEST, chart, 10,
                                   SpringLayout.EAST, charts[index - 1]);
        ((SpringLayout)this.getLayout()).putConstraint(SpringLayout.NORTH, chart, 0,
                                   SpringLayout.NORTH, charts[index - 1]);
    }

    /**
     * Checks the option selected by the user in the normalize check box.
     * If normalize is selected, computes the normalized expression chart
     * and displays it. If normalize is not selected, computes original
     * expression chart and displays it.
     *
     * @param e the <code>ItemEvent</code> of the check box selected state
     */
    public void normalize_itemStateChanged(ItemEvent e) {
        if (this.getComponentCount() == 2) {
            this.remove(1); // remove expression chart from the panel
        }
        this.repaintBiclusterTimePoints(normalize.isSelected());
        mainFrame.setVisible(true);
    }

    /**
     * Repaints a bicluster time-points chart, constructed with the original
     * or the normalized expression data.
     *
     * @param normalizeData the <code>boolean</code> which states if
     *                      expression data is to be normalized or not;
     *                      <code>true</code> if the data is to be normalized
     *                      <code>false</code> if the data is not to be normalized
     */
    private void repaintBiclusterTimePoints(boolean normalizeData) {
        try {
            expressionChart = expressionChart.repaintTimePointsChart(
                    chartWidth, chartHeight, normalizeData);

            this.add(expressionChart);
            ((SpringLayout)this.getLayout()).putConstraint(SpringLayout.NORTH, expressionChart,
                                       20,
                                       SpringLayout.SOUTH, normalize);
        } catch (InvalidNodeObjectTypeException ex) {
            /**
             * this exception is an internal error, it shouldn't be
             * displayed to the user;
             */
            ex.printStackTrace();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Checks the option selected by the user in the "number of
     * biclusters to display" combo box.<br/>
     * The number of biclusters selected by the user in the combo box
     * will determine how many bicluster expression chars must be displayed
     * in this panel.<br/>
     * Not all the bicluster expression charts are constructed a priori.
     * That is, only the charts to display are constructed. Others will be
     * generated later on, if the user pretends to display them.<br/>
     * Also if the user selects a number of bicluster charts to view that is
     * smaller than the number of bicluster charts that was being displayed
     * before the selection, the number of biclusters displayed is reduced
     * but the expression charts that were already constructed are maintained
     * in memory, in case the user pretends to see them again.<br/>
     * All the expression charts are destructed when the user selects another
     * node in the tree, as they are no longer needed.<br/>
     *
     * @param e the <code>ItemEvent</code> that signals the change of
     *          selection by the user in the combo box
     */
    public void option_itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.DESELECTED) {
            return;
        }

        int oldNumberOfCharts = (this.getComponentCount() - 1);
        if (oldNumberOfCharts < 0) {
            return;
        }

        int newNumberOfCharts = 12;
        switch (option.getSelectedIndex()) {
        case 0:
            newNumberOfCharts = 12;
            break;
        case 1:
            newNumberOfCharts = 24;
            break;
        case 2:
            newNumberOfCharts = 48;
            break;
        case 3:
            newNumberOfCharts = 96;
            break;
        case 4:
            newNumberOfCharts = setOfBiclusters.length;
            break;
        }

        if (newNumberOfCharts > setOfBiclusters.length) {
            newNumberOfCharts = setOfBiclusters.length;
        }

        if (oldNumberOfCharts > newNumberOfCharts) {
            for (int i = oldNumberOfCharts; i > newNumberOfCharts; i--) {
                this.remove(i);
            }
            this.setPreferredSize(new Dimension(4 * chartWidth + 3 * 10 + 40,
                                                (newNumberOfCharts / 4 + 1) *
                                                (chartHeight + 10) + 40));
            mainFrame.setVisible(true);
            return;
        }

        /**
         * newNumberOfCharts > oldNumberOfCharts
         */
        if (expressionCharts.length > oldNumberOfCharts) {
            if (newNumberOfCharts < expressionCharts.length) {
                for (int i = oldNumberOfCharts; i < newNumberOfCharts; i++) {
                    this.add(expressionCharts[i]);
                }
                this.resetSize();
                mainFrame.setVisible(true);
                return;
            }

            for (int i = oldNumberOfCharts; i < expressionCharts.length; i++) {
                this.add(expressionCharts[i]);
            }

            int currentNumberOfCharts = this.getComponentCount() - 1;
            if (currentNumberOfCharts == newNumberOfCharts) {
                this.resetSize();
                mainFrame.setVisible(true);
                return;
            }
        }

        /**
         * newNumberOfCharts (to display) > oldNumberOfCharts (displayed)
         * &&
         * currentNumberOfCharts (displayed) == charts.length (chart objects created)
         * currentNumberOfCharts (displayed) < newNumberOfCharts (to display)
         */
        PanelExpressionChart[] newPanelExpressionCharts = new
                PanelExpressionChart[newNumberOfCharts];
        int i;
        for (i = 0; i < expressionCharts.length; i++) {
            newPanelExpressionCharts[i] = expressionCharts[i];
        }

        boolean postProcessed = expressionCharts[0].getChart().
                                getTitle().getText().
                                startsWith("Post");
        expressionCharts = newPanelExpressionCharts;
        if (expressionType == BICLUSTER_TIME_POINTS) {
            try {
                if (postProcessed) {
                    for (int k = i; k < newNumberOfCharts; k++) {
                        Bicluster bicluster = (Bicluster)
                                              setOfBiclusters[k];
                        expressionCharts[k] = PanelExpressionChart.
                                              createExpressionBiclusterTimePointsChart(
                                bicluster,
                                chartWidth,
                                chartHeight,
                                "Post-Processed " + (k + 1) +
                                "     (" + bicluster.getID() +
                                ")",
                                false,
                                true);
                        this.add(expressionCharts[k]);
                    }
                } else {
                    for (int k = i; k < newNumberOfCharts; k++) {
                        Bicluster bicluster = (Bicluster)
                                              setOfBiclusters[k];
                        expressionCharts[k] = PanelExpressionChart.
                                              createExpressionBiclusterTimePointsChart(
                                bicluster,
                                chartWidth,
                                chartHeight,
                                false,
                                true);
                        this.add(expressionCharts[k]);
                    }
                }
            }
            catch (InvalidNodeObjectTypeException ex) {
                ex.printStackTrace();
            }
            this.resetSize();
            mainFrame.setVisible(true);
            return;
        }
        else {
            /**
             * the type of charts is PATTERN
             */
            if (setOfBiclusters[0] instanceof CCC_Bicluster) {
                try {
                    if (postProcessed) {
                        for (int j = i; j < newNumberOfCharts; j++) {
                            CCC_Bicluster bicluster =
                                    (CCC_Bicluster) setOfBiclusters[j];
                            expressionCharts[j] = PanelExpressionChart.
                                                  createExpressionBiclusterPatternChart(
                                    bicluster,
                                    chartWidth,
                                    chartHeight,
                                    "Post-Processed " + (j + 1) +
                                    "     (" + bicluster.getID() +
                                    ")",
                                    true);
                            this.add(expressionCharts[j]);
                        }
                    }
                    else {
                        for (int j = i; j < newNumberOfCharts; j++) {
                            CCC_Bicluster bicluster =
                                    (CCC_Bicluster) setOfBiclusters[j];
                            expressionCharts[j] = PanelExpressionChart.
                                                  createExpressionBiclusterPatternChart(
                                    (CCC_Bicluster) setOfBiclusters[j],
                                    chartWidth,
                                    chartHeight,
                                    true);
                            this.add(expressionCharts[j]);
                        }
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                }

                this.resetSize();
                mainFrame.setVisible(true);
                return;
            }
        }
    }

    /**
     * Returns visualization options (number of biclusters to be displayed) to
     * include in a combo box when the expression type is either
     * BICLUSTER_TIME_POINTS or PATTERN and the corresponding object is a
     * group of biclusters.
     *
     * @return the set of visualization <code>String</code> options
     */
    public String[] getVisualizationOption() {
        return visualizationOption;
    }

    /**
     * Returns the type of expression for this panel's chart. It may be:<br/>
     * PanelExpression.BICLUSTER_TIME_POINTS
     * PanelExpression.ALL_TIME_POINTS
     * PanelExpression.PATTERN
     *
     * @return <code>int</code> type of expression chart in this panel
     */
    public int getExpressionType() {
        return expressionType;
    }

    /**
     * Returns the application's main frame, where this panel is being displayed.
     *
     * @return the main <code>BiggestsFrame</code>
     */
    public BiggestsFrame getMainFrame() {
        return mainFrame;
    }

    /**
     * Returns the check box which allows the user to choose between an
     * expression chart with original or normalized data. To be used on
     * BICLUSTER_TIME_POINTS and ALL_TIME_POINTS expression charts panel.
     *
     * @return normalize <code>JCheckBox</code>
     */
    public JCheckBox getNormalize() {
        return normalize;
    }

    /**
     * Returns the combo box for the user to choose the number of biclusters
     * to display. To be used when the expression type is either
     * BICLUSTER_TIME_POINTS or PATTERN and the corresponding object
     * is a group of biclusters.
     *
     * @return option <code>JComboBox</code>
     */
    public JComboBox getOption() {
        return option;
    }

    /**
     * Returns single bicluster's expression chart that is being displayed
     * in this panel. It can be a bicluster time-points, an all time-points
     * or a pattern expression chart. It may be null if this panel does
     * not contain a chart originated from an individual bicluster.
     *
     * @return <code>PanelExpressionChart</code> that holds the
     *         chart being displayed
     */
    public PanelExpressionChart getExpressionChart() {
        return expressionChart;
    }

    /**
     * Returns the expression charts of a set of biclusters (which belong to a
     * specific group of biclusters). They can be bicluster time-points charts
     * or expression pattern charts. The pointer may be null if this panel
     * does not contain charts originated from a group of biclusters.
     *
     * @return the set of <code>PanelExpressionChart</code>s
     */
    public PanelExpressionChart[] getExpressionCharts() {
        return expressionCharts;
    }

    /**
     * Returns the set of biclusters retrieved from the group of
     * biclusters to display. It may be null if this panel was not
     * constructed to display charts from a group of biclusters.
     *
     * @return the set of <code>Bicluster</code>s
     */
    public Bicluster[] getSetOfBiclusters() {
        return setOfBiclusters;
    }

    /**
     * Returns the width of the displayed expression chart(s).
     *
     * @return <code>int</code> width of expression chart(s)
     */
    public int getChartWidth() {
        return chartWidth;
    }

    /**
     * Returns the height of the displayed expression chart(s).
     *
     * @return <code>int</code> height of expression chart(s)
     */
    public int getChartHeight() {
        return chartHeight;
    }

    public void this_ancestorResized(HierarchyEvent e) {
        if (expressionCharts != null) {
            resetSize();
        }
    }

    /**
     * Resizes this panel according to its parent component's size and
     * the number of charts that is being displayed.
     */
    private void resetSize() {
        if (expressionCharts != null) {
            int width = this.getParent().getSize().width;
            Insets insets = this.getBorder().getBorderInsets(this);
            optionPanel.setPreferredSize(new Dimension(
                    width-insets.left-insets.right-20, 70));
            this.setPreferredSize(new Dimension(
                    width, (((((this.getComponentCount()-1)*200)/width))+2)
                    *(150)));
        }
        else
        if (expressionChart != null) {
            Insets insets = this.getBorder().getBorderInsets(this);
            optionPanel.setPreferredSize(new Dimension(
                    this.getSize().width-insets.left-insets.right-20, 70));
            this.setPreferredSize(new Dimension(
                    this.getSize().width,
                    mainFrame.getSize().height - 50));
        }
    }
}


class PanelExpression_this_hierarchyBoundsAdapter extends
        HierarchyBoundsAdapter {
    private PanelExpression adaptee;
    PanelExpression_this_hierarchyBoundsAdapter(PanelExpression adaptee) {
        this.adaptee = adaptee;
    }

    public void ancestorResized(HierarchyEvent e) {
        adaptee.this_ancestorResized(e);
    }
}


/**
 *
 * <p>Title: </p>
 *
 * <p>Description: </p>
 */
class PanelExpression_option_itemAdapter implements ItemListener {
    private PanelExpression adaptee;
    PanelExpression_option_itemAdapter(PanelExpression adaptee) {
        this.adaptee = adaptee;
    }

    /**
     * Handles the event fired by the selection in the option combo box.
     *
     * @param e the <code>ItemEvent</code> fired by the user selection in the option combo box
     */
    public void itemStateChanged(ItemEvent e) {
        adaptee.option_itemStateChanged(e);
    }
}


/**
 *
 * <p>Title: </p>
 *
 * <p>Description: </p>
 */
class PanelExpression_normalize_itemAdapter implements ItemListener {
    private PanelExpression adaptee;
    PanelExpression_normalize_itemAdapter(PanelExpression adaptee) {
        this.adaptee = adaptee;
    }

    /**
     * Handles the event fired by the selection in the normalize check box.
     *
     * @param e the <code>ItemEvent</code> fired by the user selection in the normalize check box
     */
    public void itemStateChanged(ItemEvent e) {
        adaptee.normalize_itemStateChanged(e);
    }
}
