package biggests.tables;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

import biggests.gui.*;
import biggests.utils.*;
import de.schlichtherle.io.*;
import smadeira.biclustering.*;
import smadeira.ontologizer.*;

/**
 * <p>Title: Table of Symbols</p>
 *
 * <p>Description:
 *                 <p>Defines a table of symbols with the following data:<br/>
 *                 - header -> conditions' names (more widely, <code>String</code>s)<br/>
 *                 - first column -> genes' names (more widely, <code>String</code>s)<br/>
 *                 - all other columns' cells -> gene expression symbols
 *                 (more widely, <code>char</code>acters)</p>
 *
 *                 <p>A table of symbols has a vertical text header and each
 *                 of its cells is colored according to the symbol that
 *                 holds in it.</p>
 *
 *                 <p>This table implements sorting functionality on
 *                 the columns, i.e., allows the reordering of the rows,
 *                 sorting them by a specific column cells' values in
 *                 ascending or descending order.</p>
 *                 </p>
 * </p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana Gonalves
 */
public class TableSymbols extends TableValues {
    /**
     * The alphabet with all discretization symbols used in the
     * discretization technique that gave rise to the set of
     * discrete values that may appear in this symbols table.
     */
    protected char[] alphabet;

    /**
     * The set of colors. Each position contains a color that
     * corresponds to the symbol of the alphabet in the same position.
     */
    protected Color[] colors;

    /**
     * Creates a new table of symbols with data from the the given node object's
     * matrix. Each one of the matrix's cells is colored according to its
     * symbol. Colors are determined according to the number of symbols of the
     * matrix's alphabet.
     *
     * @param nodeObject the <code>NodeObjectInterface</code> from which the
     *                   colored table will be created
     * @param go the <code>GOFrontEnd</code>
     * @param parent the GUI's parent <code>Component</code>
     * @throws InvalidNodeObjectTypeException when the type of the node
     *                   object is not valid
     *
     * @see #TableSymbols(Object[][] rowData, Object[] columnNames)
     * @see #constructRowData(NodeObjectInterface nodeObject)
     * @see #constructColumnNames(NodeObjectInterface nodeObject)
     */
    /*
     * A colored table looks like this:<br/>
     * ---------------------------------------------------------------------<br/>
     * | Genes Names | C1 | C2 | C3 | .....................................|<br/>
     * |-------------------------------------------------------------------|<br/>
     * | Gene1       | o  | o  | o  | .................................... |<br/>
     * | Gene2       | o  | o  | o  | .................................... |<br/>
     * | Gene3       | o  | o  | o  | .................................... |<br/>
     * | Gene4       | o  | o  | o  | .................................... |br/>
     * ---------------------------------------------------------------------<br/>
     * where 'o' represents a colored cell. Conditions' names are written with
     * vertical orientation.
     */
    public TableSymbols(NodeObjectInterface nodeObject, GOFrontEnd go,
                        Component parent) throws InvalidNodeObjectTypeException {
        this(nodeObject);
        if (nodeObject instanceof DiscretizedExpressionMatrix) {
            goProperties = ((DiscretizedExpressionMatrix) nodeObject).getGeneOntologyProperties();
        } else
        if (nodeObject instanceof Biclustering) {
            goProperties = ((IMatrix) ((Biclustering) nodeObject).getMatrix()).
                           getGeneOntologyProperties();
        } else {
            if (go != null)
                goProperties = ((Bicluster) nodeObject).
                               computeGeneOntologyTerms(go, false);
        }
        this.goFrontEnd = go;
        this.parent = parent;
    }


    /**
     * Creates a new table of symbols with data from the the given node object's
     * matrix. Each one of the matrix's cells is colored according to its
     * symbol. Colors are determined according to the number of symbols of the
     * matrix's alphabet.
     *
     * @param nodeObject the <code>NodeObjectInterface</code> from which the
     *                   colored table will be created
     * @throws InvalidNodeObjectTypeException when the type of the node
     *                   object is not valid
     *
     * @see #TableSymbols(Object[][] rowData, Object[] columnNames)
     * @see #constructRowData(NodeObjectInterface nodeObject)
     * @see #constructColumnNames(NodeObjectInterface nodeObject)
     */
    /*
     * A colored table looks like this:<br/>
     * ---------------------------------------------------------------------<br/>
     * | Genes Names | C1 | C2 | C3 | .....................................|<br/>
     * |-------------------------------------------------------------------|<br/>
     * | Gene1       | o  | o  | o  | .................................... |<br/>
     * | Gene2       | o  | o  | o  | .................................... |<br/>
     * | Gene3       | o  | o  | o  | .................................... |<br/>
     * | Gene4       | o  | o  | o  | .................................... |br/>
     * ---------------------------------------------------------------------<br/>
     * where 'o' represents a colored cell. Conditions' names are written with
     * vertical orientation.
     */
    public TableSymbols(NodeObjectInterface nodeObject) throws
            InvalidNodeObjectTypeException {
        this(constructRowData(nodeObject), constructColumnNames(nodeObject));

        if (nodeObject instanceof DiscretizedExpressionMatrix) {
            DiscretizedExpressionMatrix discretizedExpressionMatrix =
                    (DiscretizedExpressionMatrix) nodeObject;
            this.alphabet = discretizedExpressionMatrix.getAlphabet();
        } else
        if (nodeObject instanceof CCC_Bicluster) {
            CCC_Bicluster bicluster = (CCC_Bicluster) nodeObject;
            this.alphabet = bicluster.getAlphabet();
        }
        this.initializeColors();
    }

    /**
     * Creates a new table of symbols with data from the the given node object's
     * matrix. Each one of the matrix's cells is colored according to its
     * symbol. Colors are determined according to the number of symbols of the
     * matrix's alphabet.<br/>
     *
     * This class is private because it does not set the required
     * alphabet and respective colors for a table of symbols. So,
     * it is only intended to be used by the public constructor
     * of <code>TableSymbols</code><br/>
     *
     * @param rowData the <code>Object[][]</code> array with the data to fill
     *                the rows of the table
     * @param columnNames the <code>Object[][]</code> array with the data to
     *                    fill the header of the table
     * @throws InvalidNodeObjectTypeException when the type of the node
     *                   object is not valid
     * @see TableValues#TableValues(TableSorter tableSorterModel)
     * @see javax.swing.table.DefaultTableModel#DefaultTableModel(Object[][] rowData, Object[] columnNames)
     */
    /*
     * A colored table looks like this:<br/>
     * ---------------------------------------------------------------------<br/>
     * | Genes Names | C1 | C2 | C3 | .....................................|<br/>
     * |-------------------------------------------------------------------|<br/>
     * | Gene1       | o  | o  | o  | .................................... |<br/>
     * | Gene2       | o  | o  | o  | .................................... |<br/>
     * | Gene3       | o  | o  | o  | .................................... |<br/>
     * | Gene4       | o  | o  | o  | .................................... |br/>
     * ---------------------------------------------------------------------<br/>
     * where 'o' represents a colored cell. Conditions' names are written with
     * vertical orientation.
     */
    protected TableSymbols(Object[][] rowData, Object[] columnNames) {
        super(new TableSorterSymbols(new DefaultTableModel(rowData,
                columnNames)));
        int size;
        if ((rowData[0].length - 1) > 45) {
            size = 12;
            this.setRowHeight(size);
        } else {
            size = 20;
        }

        Font currentFont = this.getFont();
        Font newFont = new Font(currentFont.getFontName(),
                                currentFont.getStyle(), 9);
        this.setFont(newFont);

        /**
         * find the maximum length of all columns' names
         */
        int maxColumnNameLength = ((String) columnNames[0]).length();
        int length;
        for (Object columnName : columnNames) {
            length = ((String) columnName).length();
            if (length > maxColumnNameLength) {
                maxColumnNameLength = length;
            }
        }

        /**
         * set table's header preferences
         */
        JTableHeader header = this.getTableHeader();
        header.setReorderingAllowed(false);
        header.setPreferredSize(new Dimension(size, maxColumnNameLength * 6));
        header.setFont(newFont);

        /**
         * set table header columns' preferences
         */
        Enumeration<TableColumn>
                tableColumns = header.getColumnModel().getColumns();
        header.setResizingAllowed(true);
        if (tableColumns.hasMoreElements()) {
            tableColumns.nextElement().setResizable(true);
        } while (tableColumns.hasMoreElements()) {
            tableColumns.nextElement().setResizable(false);
        }

        /**
         * set columns' preferred width
         */
        tableColumns = this.getColumnModel().getColumns();
        if (tableColumns.hasMoreElements()) {
            tableColumns.nextElement().setPreferredWidth(70);
        } while (tableColumns.hasMoreElements()) {
            tableColumns.nextElement().setPreferredWidth(size);
        }
    }

    /**
     * Initializes the array of colors to be used by prepareRenderer to
     * color cells, when the table has to be displayed. The <code>Color</code>
     * in <code>colors[i]</code> is the color that will fill the background
     * of the cells that contain the char symbol in <code>alphabet[i]</code>.
     */
    protected void initializeColors() {
        int interval;
        int middleIndex;
        int fraction;

        this.colors = new Color[this.alphabet.length];

        /**
         * get the middle index of the alphabet's array
         * this value is also:
         * the number of symbols that exist on each side
         * of the middle element (if the length of the alphabet's array is odd);
         * or
         * the exact half of the number of symbols in the alphabet (if the
         * length of the alphabet's array is even)
         */
        middleIndex = this.alphabet.length / 2;

        /**
         * calculate the interval, i.e., the increment value that will
         * differenciate the color red or green component value
         */
        interval = 255 / middleIndex;

        /**
         * set the green colors, for the symbols with lower ASCII codes, which
         * represent lower gene expression values
         */
        fraction = 255;
        for (int i = 0; i < middleIndex; i++) {
            this.colors[i] = new Color(0, fraction, 0);
            fraction -= interval;
        }

        if ((this.alphabet.length % 2) != 0) {
            /**
             * if number of symbols is odd, then the symbol in the
             * middle gets the color black
             */
            this.colors[middleIndex] = Color.black;

            /**
             * since the number of symbols is odd and the middle value
             * is colored black, red colored symbols are on its right side
             * set the red colors for the symbols with higher ASCII codes,
             * which represent higher gene expression values
             */
            fraction += interval;
            for (int i = middleIndex + 1; i < this.alphabet.length; i++) {
                this.colors[i] = new Color(fraction, 0, 0);
                fraction += interval;
            }
            return;
        }

        /**
         * the number of symbols is even, so there is no black middle value
         * set the red colors for the symbols with higher ASCII codes,
         * which represent higher gene expression values
         */
        fraction += interval;
        for (int i = middleIndex; i < this.alphabet.length; i++) {
            this.colors[i] = new Color(fraction, 0, 0);
            fraction += interval;
        }
    }

    /**
     * Overrides <code>TableValues</code>'s <code>prepareRenderer</code> method.
     * Prepares the renderer by querying the data model for the value and
     * selection state of the cell at <code>row</code>, <code>column</code>.
     * Returns the component (may be a <code>Component</code>
     * or a <code>JComponent</code>) under the event location.
     *
     * @param renderer  the <code>TableCellRenderer</code> to prepare
     * @param row       the row of the cell to render, where 0 is the first row
     * @param column    the column of the cell to render,
     *			where 0 is the first column
     * @return          the <code>Component</code> under the event location
     *
     * @see TableValues#prepareRenderer(TableCellRenderer renderer, int row, int column)
     */
    public Component prepareRenderer(TableCellRenderer renderer, int row,
                                     int column) {
        Component component = super.prepareRenderer(renderer, row, column);

        if (column != 0) {
            /**
             * this cell is not on the first column, i.e., contains a gene
             * expression symbol (not a gene name)
             */
            char symbol = ((Character)this.getValueAt(row, column)).charValue();
            for (int i = 0; i < this.alphabet.length; i++) {
                if (this.alphabet[i] == symbol) {
                    component.setBackground(this.colors[i]);
                    component.setForeground(this.colors[i]);
                    break;
                }
            }
        } else {
            if (!isCellSelected(row, column)) {
                /**
                 * this cell is on the first column and it is not selected by
                 * the user; so, the background is set to default
                 */
                component.setBackground(getBackground());
                component.setForeground(getForeground());
            }
        }
        return component;
    }

    /**
     * Constructs the row data matrix with symbolic data retrieved
     * from the node object's symbolic matrix.<br/>
     * <p>
     * The final structure of the row data is:<br/>
     * - the first column corresponds to the names of the genes<br/>
     * - the other cells of the matrix contain char values, which are the gene
     *   expression symbols under certain conditions
     * </p>
     * <p>
     * <code>protected</code>: it's intended to be used only by the
     *                       <code>TableSymbols</code> constructor and
     *                       constructors of <code>TableSymbols</code>'s
     *                       subclasses.<br/>
     * <code>static</code>: because object <code>this</code> cannot be
     *                      referrenced by the constructor before it is
     *                      actually created.
     * </p>
     * <p>
     * Valid node object classes are:<br/>
     *       <code>DiscretizedExpressionMatrix</code><br/>
     *       <code>CCC_Bicluster</code><br/>
     *       <code>E_CCC_Bicluster</code>
     * </p>
     *
     * @param nodeObject the <code>NodeObjectInterface</code> which provides the
     *                   data needed to create the row data matrix
     * @return the <code>Object[][]</code> row data matrix, ready to be used by
     *         <code>TableValues</code>'s constructor
     * @throws InvalidNodeObjectTypeException when the type of the node object is not valid
     */
    protected static Object[][] constructRowData(NodeObjectInterface nodeObject) throws
            InvalidNodeObjectTypeException {
        String[] geneNames = null;
        String[] conditionNames = null;
        char[][] symbolsMatrix = null;
        Object[][] rowData = null;

        if (nodeObject instanceof DiscretizedExpressionMatrix) {
            /**
             * node object is a discretized expression matrix
             */
            DiscretizedExpressionMatrix discretizedExpressionMatrix =
                    ((DiscretizedExpressionMatrix) nodeObject);

            /**
             * get gene and condition names
             */
            geneNames = discretizedExpressionMatrix.getOriginalExpressionMatrix().
                        getGenesNames();
            conditionNames = discretizedExpressionMatrix.getConditionsNames();

            /**
             * get the symbolic expression matrix
             */
            symbolsMatrix = discretizedExpressionMatrix.
                            getSymbolicExpressionMatrix();
        } else
        if ((nodeObject instanceof CCC_Bicluster) ||
            (nodeObject instanceof E_CCC_Bicluster)) {
            /**
             * node object is a bicluster or a post-processed bicluster with
             * symbolic data
             */
            CCC_Bicluster bicluster = (CCC_Bicluster) nodeObject;

            /**
             * get gene and condition names
             */
            geneNames = bicluster.getGenesNames();
            conditionNames = bicluster.getConditionsNames();

            /**
             * get the bicluster/post-processed bicluster symbolic matrix
             */
            symbolsMatrix = bicluster.getBiclusterSymbolicMatrix();
        } else {
            throw (new InvalidNodeObjectTypeException("Only discretized " +
                    "matrices and individual biclusters or post-processed " +
                    "biclusters that were constructer using discretized " +
                    "matrices are allowed."));
        }

        /**
         * construct row data matrix:
         * - first column cells have the gene names
         * - all other column cells contain gene expression symbols
         */
        if (symbolsMatrix[0].length == conditionNames.length - 1) {
            /**
             * this table corresponds to a bicluster which was constructed
             * from a discretized VBTP matrix; so, the names of the conditions
             * are one more than the number of data columns
             */
            rowData = new Object[geneNames.length][conditionNames.length];
            for (int i = 0; i < geneNames.length; i++) {
                rowData[i][0] = geneNames[i];
                for (int j = 1; j < conditionNames.length; j++) {
                    rowData[i][j] = symbolsMatrix[i][j - 1];
                }
            }
            return rowData;
        }

        rowData = new Object[geneNames.length][conditionNames.length + 1];
        for (int i = 0; i < geneNames.length; i++) {
            rowData[i][0] = geneNames[i];
            for (int j = 1; j < conditionNames.length + 1; j++) {
                rowData[i][j] = symbolsMatrix[i][j - 1];
            }
        }
        return rowData;
    }

    /**
     * Constructs the column names' array with symbolic data retrieved
     * from the node object's symbolic matrix.<br/>
     * <p>
     * The final structure of the column names' array is:<br/>
     * - the first cell contains the gene column name<br/>
     * - the remaining cells contain the condition names.
     * </p>
     * <p>
     * <code>protected</code>: it's intended to be used only by the
     *                       <code>TableValues</code>'s constructor and
     *                       <code>TableValues</code> subclasses'
     *                       constructors.<br/>
     * <code>static</code>: because object <code>this</code> cannot be
     *                      referrenced by the constructor before it is
     *                      actually created.
     * </p>
     * <p>
     * Valid node object classes are:<br/>
     *       <code>DiscretizedExpressionMatrix</code><br/>
     *       <code>CCC_Bicluster</code><br/>
     *       <code>E_CCC_Bicluster</code>
     * </p>
     *
     * @param nodeObject the <code>NodeObjectInterface</code> which provides the
     *                   data needed to create the column names' array
     * @return the <code>Object[]</code> array with the column names
     * @throws InvalidNodeObjectTypeException when the type of the node object is not valid
     */
    protected static Object[] constructColumnNames(NodeObjectInterface
            nodeObject) throws InvalidNodeObjectTypeException {
        String[] conditionNames = null;

        if (nodeObject instanceof DiscretizedExpressionMatrix) {
            /**
             * node object is a discretized expression matrix
             */
            DiscretizedExpressionMatrix discretizedExpressionMatrix = (
                    DiscretizedExpressionMatrix) nodeObject;

            /**
             * get the conditions' names of the expression (or preprocessed
             * expression) matrix from which the discretized matrix was derived
             */
            conditionNames = discretizedExpressionMatrix.getConditionsNames();
        } else
        if ((nodeObject instanceof CCC_Bicluster) ||
            (nodeObject instanceof E_CCC_Bicluster)) {
            /**
             * node object is a bicluster or a post-processed bicluster with
             * symbolic data
             */
            CCC_Bicluster bicluster = (CCC_Bicluster) nodeObject;

            /**
             * get the names of the conditions in the bicluster
             */
            conditionNames = bicluster.getConditionsNames();

            int[] conditionIndexes = bicluster.getColumnsIndexes();

            if (bicluster.getBiclusterSymbolicMatrix()[0].length ==
                conditionNames.length - 1) {
                /**
                 * this table corresponds to a bicluster constructed from a
                 * discretized VBTP matrix
                 */
                String[] columnNames = new String[conditionNames.length];
                columnNames[0] = "Genes Names";
                for (int i = 1; i < columnNames.length; i++) {
                    columnNames[i] = "TP" + conditionIndexes[i - 1] + "_TP" +
                                     conditionIndexes[i];
                }
                return columnNames;

            }
        } else {
            throw (new InvalidNodeObjectTypeException("Only discretized " +
                    "matrices and individual biclusters or post-processed " +
                    "biclusters that were constructer using discretized " +
                    "matrices are allowed."));
        }

        /**
         * construct the column names for this table:
         * - first cell contains a label for the first column (the column which
         * holds the genes' names) - "Genes Names"
         * - each of the other cells contain the name of the corresponding
         * condition
         */
        String[] columnNames = new String[conditionNames.length + 1];

        columnNames[0] = "Genes Names";
        for (int i = 1; i < columnNames.length; i++) {
            columnNames[i] = conditionNames[i - 1];
        }
        return columnNames;
    }

    /**
     * Exports this table to an image file. Allows the user to select
     * the image's filepath in a file chooser dialog and choose the image
     * format (PNG or JPG).
     */
    public void exportToImage() {
        if (parent == null) {
            return;
        }

        JFileChooser pathFileChooser = new JFileChooser();
        try {
            /**
             * initialize the file chooser with the intended file filters
             * (one for png files, and another for svg files)
             */
            if (parent instanceof BiggestsFrame) {
                pathFileChooser.addChoosableFileFilter(new FileFilterDirectory(
                        ((BiggestsFrame) parent).getPathsToFolders(), ".png"));
                pathFileChooser.addChoosableFileFilter(new FileFilterDirectory(
                        ((BiggestsFrame) parent).getPathsToFolders(), ".jpg"));
            } else {
                pathFileChooser.addChoosableFileFilter(new FileFilterExtension(
                        ".png"));
                pathFileChooser.addChoosableFileFilter(new FileFilterExtension(
                        ".jpg"));
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        /**
         * turn off accept all file filter
         */
        pathFileChooser.setAcceptAllFileFilterUsed(false);

        /**
         * show the save dialog to the user
         */
        int option = pathFileChooser.showSaveDialog(parent);

        if (option == JFileChooser.APPROVE_OPTION) {
            /**
             * user hit approve option; get the specified path for the file
             */
            String newPath = pathFileChooser.getSelectedFile().getPath();
            String extension = "";
            if (parent instanceof BiggestsFrame) {
                extension = ((FileFilterDirectory) pathFileChooser.
                             getFileFilter()).getValidFileExtension();
                newPath = ((FileFilterDirectory) pathFileChooser.
                           getFileFilter()).checkAndAddFileExtension(newPath);
            } else {
                extension = ((FileFilterExtension) pathFileChooser.
                             getFileFilter()).getValidFileExtension();
                newPath = ((FileFilterExtension) pathFileChooser.
                           getFileFilter()).checkAndAddFileExtension(newPath);
            }
            File newFile = new File(newPath);
            if (newFile.exists()) {
                if (newFile.isDirectory()) {
                    JOptionPane.showMessageDialog(parent,
                                                  "Could not save image." +
                                                  "\nYou didn't specify a filename.",
                                                  "Filename missing",
                                                  JOptionPane.ERROR_MESSAGE);
                    return;
                }
                if (newFile.isFile()) {
                    int opt = JOptionPane.showOptionDialog(parent,
                            "The specified file already exists." +
                            "\nDo you want to overwrite it?",
                            "Filename already exists",
                            JOptionPane.YES_NO_OPTION,
                            JOptionPane.QUESTION_MESSAGE,
                            null, null, JOptionPane.YES_OPTION);
                    if (opt == JOptionPane.NO_OPTION) {
                        /**
                         * the user does not want to overwrite the file
                         */
                        return;
                    }
                }
            }

            try {
                int locationX = this.getTableHeader().getLocation().x;
                int locationY = this.getTableHeader().getLocation().y;
                int borderWidth = 0;
                Color oldColor = this.getBackground();
                this.setBackground(Color.white);
                this.getTableHeader().setBackground(Color.white);
                this.getParent().setBackground(Color.white);
                if (this.getParent() instanceof JComponent) {
                    Border border = ((JComponent)this.getParent()).getBorder();
                    if (border != null) {
                        borderWidth = border.getBorderInsets(this.getParent()).
                                      left +
                                      border.getBorderInsets(this.getParent()).
                                      right;
                    }
                }
                try {
                    BiggestsIO.exportComponentToImage(this.getParent(),
                            locationX, locationY,
                            this.getWidth() + borderWidth,
                            this.getParent().getHeight() - locationY,
                            extension, newPath);
                } catch (Exception ex) {
                    BiggestsIO.exportComponentToImage(this.getParent(),
                            extension, newPath);
                }
                this.setBackground(oldColor);
                this.getTableHeader().setBackground(oldColor);
                this.getParent().setBackground(oldColor);
            } catch (Exception ex) {
                ex.printStackTrace();
                JOptionPane.showMessageDialog(parent, "Couldn't export " +
                                              "expression matrix as image.",
                                              "Export as image failed",
                                              JOptionPane.ERROR_MESSAGE);
            }
        }
    }


    /**
     * Returns the alphabet with all discretization symbols used in the
     * discretization technique that gave rise to the set of
     * discrete values that may appear in this symbols table.
     *
     * @return <code>char[]</code> the set of symbols of discretization
     */
    public char[] getAlphabet() {
        return alphabet;
    }

    /**
     * Returns the set of colors used to color each of the table's cells.
     * Each position contains a color that
     * corresponds to the symbol of the alphabet in the same position.
     *
     * @return <code>Color[]</code> the set of <code>Color</code>s that
     *         correspond to the respective symbols of the alphabet
     */
    public Color[] getColors() {
        return colors;
    }

    /**
     * Handles a mouse click in this table of colors. If the click is
     * in the mouse's left button, calls <code>TableValues</code>
     * <code>this_mouseClicked</code> method to display the terms
     * annotated with the gene in the clicked row of the table.
     *
     * If the click is in the mouse's right button, shows 'Export to image'
     * popup menu.
     *
     * @param e the <code>MouseEvent</code> fired when the user clicks this table
     */
    public void this_mouseClicked(MouseEvent e) {
        if (e.getButton() == MouseEvent.BUTTON1) {
            super.this_mouseClicked(e);
        } else {
            JPopupMenu popup = new JPopupMenu();
            JMenuItem item = new JMenuItem("Export as image");
            item.addActionListener(new ItemHandler());
            popup.add(item);
            popup.show(this, e.getPoint().getLocation().x,
                       e.getPoint().getLocation().y);
        }
    }

    /**
     * <p>Description: Implements an ActionListener for 'Export as image'
     *                 context menu item.
     * </p>
     */
    private class ItemHandler implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            exportToImage();
        }
    }
}
