package biggests.tables;

import java.awt.Color;
import smadeira.biclustering.Biclustering;
import smadeira.biclustering.Bicluster;
import smadeira.ontologizer.GOFrontEnd;
import biggests.utils.InvalidNodeObjectTypeException;
import java.awt.Component;
import smadeira.biclustering.DiscretizedExpressionMatrix;
import javax.swing.table.TableCellRenderer;
import smadeira.biclustering.CCC_Bicluster;

/**
 * <p>Title: Table of Symbols with Missing Values</p>
 *
 * <p>Description:
 *                 <p>Defines a table of symbols with the following data:<br/>
 *                 - header -> conditions' names (more widely, <code>String</code>s)<br/>
 *                 - first column -> genes' names (more widely, <code>String</code>s)<br/>
 *                 - all other columns' cells -> gene expression symbols
 *                 (more widely, <code>char</code>acters)</p>
 *
 *                 <p>A table of symbols has a vertical text header and each
 *                 of its cells is colored according to the symbol that
 *                 holds in it.</p>
 *
 *                 <p>This table implements sorting functionality on
 *                 the columns, i.e., allows the reordering of the rows,
 *                 sorting them by tha values of a given column in
 *                 ascending or descending order.</p>
 *                 </p>
 *                 <p>Specific for data with missing values, that is, some
 *                 erroneus values that are not part of the original data
 *                 and are marked by some <code>char</code> value.</p>
 * </p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana P. Gonalves
 */
public class TableSymbolsWithMissingValues extends TableSymbols {
    /**
     * The value for marking missings.
     */
    private char missingValue;

    /**
     * Creates a new table of symbols with data from the the given
     * discretized expression matrix. Each one of the matrix's cells
     * is colored according to its symbol. Colors are determined
     * according to the number of symbols of the matrix's alphabet.
     * This table also displays the terms annotated with each gene
     * (when a specific table's row is clicked by the user).
     *
     * @param discreteMatrix the <code>DiscretizedExpressionMatrix</code>
     *                   from which the colored table will be created
     * @param go the <code>GOFrontEnd</code>
     * @param parent the GUI's parent <code>Component</code>
     * @throws InvalidNodeObjectTypeException if the type of the node
     *                   object is not valid
     *
     * @see #TableSymbols(Object[][] rowData, Object[] columnNames)
     * @see #constructRowData(NodeObjectInterface nodeObject)
     * @see #constructColumnNames(NodeObjectInterface nodeObject)
     */
    /*
     * A colored table looks like this:<br/>
     * ---------------------------------------------------------------------<br/>
     * | Genes Names | C1 | C2 | C3 | .....................................|<br/>
     * |-------------------------------------------------------------------|<br/>
     * | Gene1       | o  | o  | o  | .................................... |<br/>
     * | Gene2       | o  | o  | o  | .................................... |<br/>
     * | Gene3       | o  | o  | o  | .................................... |<br/>
     * | Gene4       | o  | o  | o  | .................................... |br/>
     * ---------------------------------------------------------------------<br/>
     * where 'o' represents a colored cell. Conditions' names are written with
     * vertical orientation.
     */
    public TableSymbolsWithMissingValues(DiscretizedExpressionMatrix discreteMatrix, GOFrontEnd go,
                        Component parent) throws InvalidNodeObjectTypeException {
        this(discreteMatrix);
        goProperties = discreteMatrix.getGeneOntologyProperties();
        this.goFrontEnd = go;
        this.parent = parent;
    }


    /**
     * Creates a new table of symbols with data from the the given
     * discretized expression matrix. Each one of the matrix's cells
     * is colored according to its symbol. Colors are determined
     * according to the number of symbols of the matrix's alphabet.
     *
     * @param discreteMatrix the <code>DiscretizedExpressionMatrix</code>
     *                   from which the colored table will be created
     * @throws InvalidNodeObjectTypeException when the type of the node
     *                   object is not valid
     *
     * @see #TableSymbols(Object[][] rowData, Object[] columnNames)
     * @see #constructRowData(NodeObjectInterface nodeObject)
     * @see #constructColumnNames(NodeObjectInterface nodeObject)
     */
    /*
     * A colored table looks like this:<br/>
     * ---------------------------------------------------------------------<br/>
     * | Genes Names | C1 | C2 | C3 | .....................................|<br/>
     * |-------------------------------------------------------------------|<br/>
     * | Gene1       | o  | o  | o  | .................................... |<br/>
     * | Gene2       | o  | o  | o  | .................................... |<br/>
     * | Gene3       | o  | o  | o  | .................................... |<br/>
     * | Gene4       | o  | o  | o  | .................................... |br/>
     * ---------------------------------------------------------------------<br/>
     * where 'o' represents a colored cell. Conditions' names are written with
     * vertical orientation.
     */
    public TableSymbolsWithMissingValues(DiscretizedExpressionMatrix discreteMatrix) throws
            InvalidNodeObjectTypeException {
        super(constructRowData(discreteMatrix), constructColumnNames(discreteMatrix));
        this.missingValue = discreteMatrix.getMissingValue();
        this.alphabet = discreteMatrix.getAlphabet();
        this.initializeColors();
    }

    /**
     * Creates a new table of symbols with data from the the given bicluster.
     * Each one of the matrix's cells is colored according to its
     * symbol. Colors are determined according to the number of symbols of the
     * bicluster's alphabet.
     * This table also displays the terms annotated with each gene
     * (when a specific table's row is clicked by the user).
     *
     * @param bicluster the <code>Bicluster</code> from which the
     *                   colored table will be created
     * @param go the <code>GOFrontEnd</code>
     * @param parent the GUI's parent <code>Component</code>
     * @throws InvalidNodeObjectTypeException when the type of the node
     *                   object is not valid
     *
     * @see #TableSymbols(Object[][] rowData, Object[] columnNames)
     * @see #constructRowData(NodeObjectInterface nodeObject)
     * @see #constructColumnNames(NodeObjectInterface nodeObject)
     */
    /*
     * A colored table looks like this:<br/>
     * ---------------------------------------------------------------------<br/>
     * | Genes Names | C1 | C2 | C3 | .....................................|<br/>
     * |-------------------------------------------------------------------|<br/>
     * | Gene1       | o  | o  | o  | .................................... |<br/>
     * | Gene2       | o  | o  | o  | .................................... |<br/>
     * | Gene3       | o  | o  | o  | .................................... |<br/>
     * | Gene4       | o  | o  | o  | .................................... |br/>
     * ---------------------------------------------------------------------<br/>
     * where 'o' represents a colored cell. Conditions' names are written with
     * vertical orientation.
     */
    public TableSymbolsWithMissingValues(CCC_Bicluster bicluster, GOFrontEnd go,
                        Component parent) throws InvalidNodeObjectTypeException {
        this(bicluster);
        if (go != null) {
            goProperties = bicluster.computeGeneOntologyTerms(go, false);
            this.goFrontEnd = go;
        }
        this.parent = parent;
    }


    /**
     * Creates a new table of symbols with data from the given bicluster's
     * matrix. Each one of the matrix's cells is colored according to its
     * symbol. Colors are determined according to the number of symbols of the
     * bicluster's alphabet.
     *
     * @param bicluster the <code>Bicluster</code> from which the
     *                   colored table will be created
     * @throws InvalidNodeObjectTypeException when the type of the node
     *                   object is not valid
     *
     * @see #TableSymbols(Object[][] rowData, Object[] columnNames)
     * @see #constructRowData(NodeObjectInterface nodeObject)
     * @see #constructColumnNames(NodeObjectInterface nodeObject)
     */
    /*
     * A colored table looks like this:<br/>
     * ---------------------------------------------------------------------<br/>
     * | Genes Names | C1 | C2 | C3 | .....................................|<br/>
     * |-------------------------------------------------------------------|<br/>
     * | Gene1       | o  | o  | o  | .................................... |<br/>
     * | Gene2       | o  | o  | o  | .................................... |<br/>
     * | Gene3       | o  | o  | o  | .................................... |<br/>
     * | Gene4       | o  | o  | o  | .................................... |br/>
     * ---------------------------------------------------------------------<br/>
     * where 'o' represents a colored cell. Conditions' names are written with
     * vertical orientation.
     */
    public TableSymbolsWithMissingValues(CCC_Bicluster bicluster) throws
            InvalidNodeObjectTypeException {
        super(constructRowData(bicluster), constructColumnNames(bicluster));
        this.missingValue = ((DiscretizedExpressionMatrix)bicluster.
                             getBiclusterSource().getMatrix()).
                            getMissingValue();
        this.alphabet = bicluster.getAlphabet();
        this.initializeColors();
    }

    /**
     * Overrides <code>TableValues</code>'s <code>prepareRenderer</code> method.
     * Prepares the renderer by querying the data model for the value and
     * selection state of the cell at <code>row</code>, <code>column</code>.
     * Returns the component (may be a <code>Component</code>
     * or a <code>JComponent</code>) under the event location.
     *
     * @param renderer  the <code>TableCellRenderer</code> to prepare
     * @param row       the row of the cell to render, where 0 is the first row
     * @param column    the column of the cell to render,
     *			where 0 is the first column
     * @return          the <code>Component</code> under the event location
     *
     * @see TableValues#prepareRenderer(TableCellRenderer renderer, int row, int column)
     */
    public Component prepareRenderer(TableCellRenderer renderer, int row,
                                     int column) {
        Component component = super.prepareRenderer(renderer, row, column);

        if (column != 0) {
            /**
             * this cell is not on the first column, i.e., contains a gene
             * expression symbol (not a gene name)
             */
            char symbol = ((Character)this.getValueAt(row, column)).charValue();
            for (int i = 0; i < this.alphabet.length; i++) {
                if (symbol == missingValue) {
                    component.setBackground(Color.yellow);
                    component.setForeground(Color.yellow);
                }
                if (this.alphabet[i] == symbol) {
                    component.setBackground(this.colors[i]);
                    component.setForeground(this.colors[i]);
                    break;
                }
            }
        } else {
            if (!isCellSelected(row, column)) {
                /**
                 * this cell is on the first column and it is not selected by
                 * the user; so, the background is set to default
                 */
                component.setBackground(getBackground());
                component.setForeground(getForeground());
            }
        }
        return component;
    }

    /**
     * Constructs the row data matrix with symbolic data retrieved
     * from the node object's symbolic matrix.<br/>
     * <p>
     * The final structure of the row data is:<br/>
     * - the first column corresponds to the names of the genes<br/>
     * - the other cells of the matrix contain char values, which are the gene
     *   expression symbols under certain conditions
     * </p>
     *
     * @param discreteMatrix the <code>DiscretizedExpressionMatrix</code>
     *                   which provides the data needed to create the
     *                   row data matrix
     * @return the <code>Object[][]</code> row data matrix, ready to be used by
     *         <code>TableValues</code>'s constructor
     * @throws InvalidNodeObjectTypeException when the type of the node object is not valid
     */
    protected static Object[][] constructRowData(DiscretizedExpressionMatrix discreteMatrix) throws
            InvalidNodeObjectTypeException {
        String[] geneNames = null;
        String[] conditionNames = null;
        char[][] symbolsMatrix = null;
        Object[][] rowData = null;

        /**
         * get gene and condition names
         */
        geneNames = discreteMatrix.getOriginalExpressionMatrix().
                    getGenesNames();
        conditionNames = discreteMatrix.getConditionsNames();

        /**
         * get the symbolic expression matrix
         */
        symbolsMatrix = discreteMatrix.
                        getSymbolicExpressionMatrix();

        /**
         * construct row data matrix:
         * - first column cells have the gene names
         * - all other column cells contain gene expression symbols
         */
        if (symbolsMatrix[0].length == conditionNames.length - 1) {
            /**
             * this table corresponds to a bicluster which was constructed
             * from a discretized VBTP matrix; so, the names of the conditions
             * are one more than the number of data columns
             */
            rowData = new Object[geneNames.length][conditionNames.length];
            for (int i = 0; i < geneNames.length; i++) {
                rowData[i][0] = geneNames[i];
                for (int j = 1; j < conditionNames.length; j++) {
                    rowData[i][j] = symbolsMatrix[i][j - 1];
                }
            }
            return rowData;
        }

        rowData = new Object[geneNames.length][conditionNames.length + 1];
        for (int i = 0; i < geneNames.length; i++) {
            rowData[i][0] = geneNames[i];
            for (int j = 1; j < conditionNames.length + 1; j++) {
                rowData[i][j] = symbolsMatrix[i][j - 1];
            }
        }
        return rowData;
    }

    /**
     * Constructs the column names' array with symbolic data retrieved
     * from the given discretized expression matrix.<br/>
     * <p>
     * The final structure of the column names' array is:<br/>
     * - the first cell contains the gene column name<br/>
     * - the remaining cells contain the condition names.
     * </p>
     *
     * @param discreteMatrix the <code>DiscretizedExpressionMatrix</code>
     *                   which provides the data needed to create the
     *                   column names' array
     * @return the <code>Object[]</code> array with the column names
     * @throws InvalidNodeObjectTypeException when the type of the node object is not valid
     */
    protected static Object[] constructColumnNames(DiscretizedExpressionMatrix
            discreteMatrix) throws InvalidNodeObjectTypeException {
        String[] conditionNames = null;

        /**
         * get the conditions' names of the expression (or preprocessed
         * expression) matrix from which the discretized matrix was derived
         */
        conditionNames = discreteMatrix.getConditionsNames();

        /**
         * construct the column names for this table:
         * - first cell contains a label for the first column (the column which
         * holds the genes' names) - "Genes Names"
         * - each of the other cells contain the name of the corresponding
         * condition
         */
        String[] columnNames = new String[conditionNames.length + 1];

        columnNames[0] = "Genes Names";
        for (int i = 1; i < columnNames.length; i++) {
            columnNames[i] = conditionNames[i - 1];
        }
        return columnNames;
    }
}
