package biggests.tree;

import java.io.*;

import biggests.utils.*;

/**
 * <p>Title: Dataset</p>
 *
 * <p>Description: Defines a dataset information object, which holds all the
 *                 relevant information about a specific dataset loaded.</p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana Gonalves
 */

public class Dataset implements Serializable {
    public static final long serialVersionUID = -9150182590356849832L;

    /**
     * The name of the dataset.
     */
    private String name;

    /**
     * The id value of the dataset.
     */
    private int id;

    /**
     * The last id given to a preprocessed node information
     * object within the current dataset.
     */
    private int lastPreprocessedId;

    /**
     * The last id given to a discretized node information object
     * within the current dataset.
     */
    private int lastDiscretizedId;

    /**
     * The last id given to a bicluster group node information object
     * within the current dataset.
     */
    private int lastBiclusterGroupId;

    /**
     * The last id given to a post-processed group node information
     * object within the current dataset.
     */
    private int lastPostprocessedGroupId;

    /**
     * Constructs a new <code>Dataset</code> object with the given id.
     *
     * @param id int the given id
     */
    public Dataset(int id) {
        this.name = "Dataset";
        this.id = id;
        lastPreprocessedId = -1;
        lastDiscretizedId = -1;
        lastBiclusterGroupId = -1;
        lastPostprocessedGroupId = -1;
    }

    /**
     * Generates a new id value (within this dataset) for a new node
     * information object with the specified data type.
     *
     * @param nodeObjectType int the type of the node object:<br/>
     *      <code>NodeInfo.PREPROCESSED</code><br/>
     *      <code>NodeInfo.DISCRETIZED</code><br/>
     *      <code>NodeInfo.BICLUSTER_GROUP</code><br/>
     *      <code>NodeInfo.POSTPROCESSED_GROUP</code><br/>
     * @param update <code>boolean</code> <code>true</code> to update the
     *               number of nodes created in the tree for this
     *               <code>nodeObjectType</code> so far; <code>false</code>
     *               if one only wants to know which id is next, without
     *               inserting a new node in the tree
     * @return int the new id value
     * @throws InvalidNodeObjectTypeException when the node object type is not valid
     */
    public int generateNewNodeObjectTypeId(int nodeObjectType, boolean update) throws
            InvalidNodeObjectTypeException {
        /**
         * calculate the new id based on the type of the node object
         */
        if (update) {
            switch (nodeObjectType) {
            case NodeInfo.PREPROCESSED:
                lastPreprocessedId++;
                return (lastPreprocessedId);
            case NodeInfo.DISCRETIZED:
                lastDiscretizedId++;
                return (lastDiscretizedId);
            case NodeInfo.BICLUSTER_GROUP:
                lastBiclusterGroupId++;
                return (lastBiclusterGroupId);
            case NodeInfo.POSTPROCESSED_GROUP:
                lastPostprocessedGroupId++;
                return (lastPostprocessedGroupId);
            default:
                throw (new InvalidNodeObjectTypeException(
                        "Only preprocessed, discretized, bicluster groups " +
                        "and post-processed groups are allowed"));
            }
        }
        switch (nodeObjectType) {
        case NodeInfo.PREPROCESSED:
            return (lastPreprocessedId + 1);
        case NodeInfo.DISCRETIZED:
            return (lastDiscretizedId + 1);
        case NodeInfo.BICLUSTER_GROUP:
            return (lastBiclusterGroupId + 1);
        case NodeInfo.POSTPROCESSED_GROUP:
            return (lastPostprocessedGroupId + 1);
        default:
            throw (new InvalidNodeObjectTypeException(
                    "Only preprocessed, discretized, bicluster groups " +
                    "and post-processed groups are allowed"));
        }
    }

    /**
     * Returns the name of this dataset. Usually the name is only Dataset.
     *
     * @return the <code>String</code> holding the name of the dataset
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the dataset id.
     *
     * @return <code>int</code> the dataset id
     */
    public int getId() {
        return id;
    }

    /**
     * Returns the last id given to a preprocessed node information object
     * within the current dataset.
     *
     * @return <code>int</code> the last id given to a preprocessed node
     * information object within the current dataset
     */
    public int getLastPreprocessedId() {
        return lastPreprocessedId;
    }

    /**
     * Returns the last id given to a discretized node information object
     * within the current dataset.
     *
     * @return <code>int</code> the last id given to a discretized node
     * information object within the current dataset
     */
    public int getLastDiscretizedId() {
        return lastDiscretizedId;
    }

    /**
     * Returns the last id given to a bicluster group node information object
     * within the current dataset.
     *
     * @return <code>int</code> the last id given to a bicluster group node information object
     * within the current dataset
     */
    public int getLastBiclusterGroupId() {
        return lastBiclusterGroupId;
    }

    /**
     * Returns the last id given to a post-processed group node information
     * object within the current dataset.
     *
     * @return <code>int</code> the last id given to a post-processed group
     * node information object within the current dataset
     */
    public int getLastPostprocessedGroupId() {
        return lastPostprocessedGroupId;
    }

    /**
     * Overrides <code>toString()</code> method from class <code>Object</code>.
     * Returns a formatted string with the contents that will appear in
     * the tree as node name.
     *
     * @return String the dataset node name in the tree
     */
    public String toString() {
        return (name + " " + id);
    }
}
