package biggests.utils;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import biggests.gui.*;

/**
 * <p>Title: Progress Bar Frame</p>
 *
 * <p>Description: Dialog for displaying an indeterminate progress bar.</p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana Gonalves
 */
public class ProgressBarFrame extends JDialog {
    /**
     * The progress bar.
     */
    private JProgressBar progressBar;

    /**
     * Message history of computation's progress.
     */
    private JTextArea messageHistory;

    /**
     * Abort/Cancel button.
     */
    private JButton cancelButton;

    /**
     * Working thread.
     */
    private Thread workingThread;

    /**
     * Creates a new <code>ProgressBarFrame</code> dialog.
     *
     * @param parent the parent <code>Frame</code> of the application
     * @param workingThread Thread
     * @param initialMessage <code>String</code> the initial message of the progress bar
     * @param messageHistory <code>boolean</code> <code>true</code>
     *        if message history should be displayed, <code>false</code>
     *        otherwise
     */
    public ProgressBarFrame(Frame parent,
                            Thread workingThread,
                            String initialMessage,
                            boolean messageHistory) {
        super(parent);
        try {
            setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
            jbInit(parent, workingThread, initialMessage, messageHistory);
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * Component initialization.
     *
     * @param parent the parent <code>Frame</code> of the application
     * @param workingThread Thread
     * @param initialMessage <code>String</code> the initial message of the progress bar
     * @param messageHistory boolean
     * @throws Exception if components could not be initialized
     */
    private void jbInit(Frame parent,
                        Thread workingThread,
                        String initialMessage,
                        boolean messageHistory) throws Exception {
        this.workingThread = workingThread;

        progressBar = new JProgressBar(0, 100);
        progressBar.setString(initialMessage);
        progressBar.setStringPainted(true);
        progressBar.setIndeterminate(true);

        JPanel contentPane = (JPanel) getContentPane();
        contentPane.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
        this.addWindowListener(new ProgressBarFrame_this_windowAdapter(this));
        contentPane.add(progressBar, BorderLayout.NORTH);

        int height = 0;
        if (messageHistory) {
            this.messageHistory = new JTextArea("  " + initialMessage + "\n");
            this.messageHistory.setFont(new Font("Arial", Font.PLAIN, 11));
            JScrollPane scroll = new JScrollPane(this.messageHistory);
            contentPane.add(scroll, BorderLayout.CENTER);
            height = 200;
        }

        cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new
                                      ProgressBarFrame_cancelButton_actionAdapter(this));
        contentPane.add(cancelButton, BorderLayout.SOUTH);
        setSize(400, height + 150);

        setTitle("Progress...");
        Dimension size = this.getSize();
        Dimension parentSize = parent.getSize();
        Point loc = parent.getLocation();
        setLocation((parentSize.width - size.width) / 2 + loc.x,
                    (parentSize.height - size.height) / 2 + loc.y);
        setVisible(true);
    }

    /**
     * Returns the progress bar.
     *
     * @return JProgressBar the progress bar
     */
    public JProgressBar getProgressBar() {
        return progressBar;
    }

    /**
     * Sets the message displayed by the progress bar.
     *
     * @param s <code>String</code> the new message for the progress bar
     */
    public void setString(String s) {
        progressBar.setString(s);
        this.addMessageToMessageHistory(s);
    }

    /**
     * Adds a new message to message history.
     * @param message String
     */
    public void addMessageToMessageHistory(String message) {
        if (messageHistory != null) {
            messageHistory.setText(messageHistory.getText() + "  " + message +
                                   "\n");
        }
        this.setVisible(true);
    }

    /**
     * Returns the message displayed by the progress bar.
     *
     * @return <code>String</code> the current message displayed by the progress bar
     */
    public String getString() {
        return progressBar.getString();
    }

    /**
     * Returns message history.
     *
     * @return String
     */
    public String getMessageHistory() {
        return messageHistory.getText();
    }

    /**
     * When using the cancel button, handles a user's click on this button.
     *
     * @param e <code>ActionEvent</code> the click event
     */
    public void cancelButton_actionPerformed(ActionEvent e) {
        if (workingThread instanceof ThreadDownloadGOFiles) {
            BiggestsIO.setStop(true);
            return;
        }
        this.getParent().setEnabled(true);
        workingThread.stop();
        JOptionPane.showMessageDialog(this, "Operation aborted.",
                                      "Operation aborted",
                                      JOptionPane.WARNING_MESSAGE);
        this.dispose();
    }

    public void this_windowClosing(WindowEvent e) {

    }
}


class ProgressBarFrame_this_windowAdapter extends WindowAdapter {
    private ProgressBarFrame adaptee;
    ProgressBarFrame_this_windowAdapter(ProgressBarFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void windowClosing(WindowEvent e) {
        adaptee.this_windowClosing(e);
    }
}


/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 */
class ProgressBarFrame_cancelButton_actionAdapter implements ActionListener {
    private ProgressBarFrame adaptee;
    ProgressBarFrame_cancelButton_actionAdapter(ProgressBarFrame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.cancelButton_actionPerformed(e);
    }
}
